function [LOC_X_filtered, LOC_Y_filtered, lineDiam_lastParams, x1, y1, initLineFigure] = manualLinearSetting(LOC_X, LOC_Y, PSFSize, initLine_linearFit, initParams, automaticProcessBool)
% This function is for manually adjusting the initially fit line and the
% for setting the region to be cut out in a pop-up window (if not disabled
% with the automatic data processing). The parameters of the cut-out region
% is also asked in a pop up window. The figure is refreshed after every
% parameter modification.

% angle conversion for the user input and output
    function outputAngle=angleInConversion(InputAngle)
        outputAngle=(InputAngle)*pi/180;
    end

    function outputAngle=angleOutConversion(InputAngle)
        outputAngle=InputAngle*180/pi;
    end

%% Initialization of some parameters

xrange=initParams.xrange;
yrange=initParams.yrange;
sig=initParams.sig;
sampLineDistThreshold=initParams.sampLineDistThreshold;
nBins=initParams.nBins;
polyDegree=initParams.polyDegree;
PSFmax=initParams.PSFmax;
surfFittingPower=initParams.surfFittingPower;


% if ~isempty(initParams)
%     alpha_mean=initParams.alpha_mean;
%     linearFit.p1=tan(alpha_mean+90*pi/180);
%     linearFit.p2=mean(LOC_Y)-mean(LOC_X)*tan(alpha_mean+90*pi/180)-0*tan(alpha_mean+90*pi/180)+0;
%     
% else
    alpha_mean=atan(initLine_linearFit.m)-90*pi/180;              % angle of the fitted line
    % because the properties of the atan function
    if alpha_mean<-pi/2
        alpha_mean=alpha_mean+180*pi/180;
    elseif alpha_mean>pi/2
        alpha_mean=alpha_mean-180*pi/180;
    end
% end

dlgBool=true;                               % open later a question dialog
drawArrow = @(x,y,varargin) quiver( x(1),y(1),x(2)-x(1),y(2)-y(1),0, varargin{:} ); % drawing later the arrow showing the central displacement

% Filtering based on the PSF size
LOC_X_filtered = LOC_X(PSFSize<=PSFmax);
LOC_Y_filtered = LOC_Y(PSFSize<=PSFmax);

% the remnant parameters, the center position of the rectangular region
xCenter=mean(LOC_X_filtered);    % "x" components of the data points' mean position
yCenter=mean(LOC_Y_filtered);    % "y" components of the data points' mean position

%calculating the displacement of the fitted line (the vector pointing from [xCenter,yCenter] to the fitted line and is perpedicular to it)
% parameters of the fitted line, f(x)=m*x+h
m=initLine_linearFit.m;
h=initLine_linearFit.h;

% parameters of the line perpendicular to the fitted one, f(x)=m_perp*x+h_perp
m_perp=tan(atan(m)+pi/2);
h_perp=yCenter-m_perp*xCenter;

% the vector pointing from [xCenter,yCenter] to the fitted line and is perpedicular to it
xDispacement=(h_perp-h)/(m-m_perp)-xCenter;
yDispacement=(h/m-h_perp/m_perp)/(1/m-1/m_perp)-yCenter;
displacement=[xDispacement,yDispacement];  % magnitude of the autocorrelation's displacement

%% Setting the region to be cut out
% drawing the figure opening the dialog until the set parameters are not
% accepted
if automaticProcessBool
    %figure(10)
    initLineFigure=figure('visible','off');
else
    initLineFigure=figure();
end
set(initLineFigure, 'Name', 'Choosing the rectangular region of interest')
while dlgBool
    
    % plotting the data points and the fitted line
    ax=axes;
    scatter(LOC_X_filtered, LOC_Y_filtered, 5, 'r', '.')  % plotting the data points
    
    if exist('OCTAVE_VERSION', 'builtin') ~= 0
      % workaround for an Octave bug
      set(ax, 'XLim', [0, 1])
      set(ax, 'YLim', [0, 1])
      set(ax, 'XLimMode', 'Auto')
      set(ax, 'YLimMode', 'Auto')
    end
    hold on;
    % getting the axis limits after the scatter plot:
    xlimit=get(ax, 'XLim');
    ylimit=get(ax, 'YLim');

    % setting the axis limits back to span only the scatter plot:
    set(gca, 'XLim', xlimit, 'YLim', ylimit);
    
    % other axis formatting:
    view(90, 90)                    % exchanging the 'x' and 'y' axes
    daspect([1 1 1])                        % fixing the aspect ration, the axis('equal') and axis('image') does not work correctly
    
    % the displacement vectorstarting and ending coordinates:
    x1=[xCenter, xCenter+displacement(1)];    % starting and ending "x" coordinates of the arrow
    y1=[yCenter, yCenter+displacement(2)];    % starting and ending "y" coordinates of the arrow

    % calculate the coordiantes of the rectangle to be plotted:
    x=zeros(5,1);               % "x" coordinates of the vertices of the recttangle bounding the displacement region
    x(1)=x1(2)+(cos(alpha_mean+90*pi/180)*(-xrange)-sin(alpha_mean+90*pi/180)*(+yrange));
    x(2)=x1(2)+(cos(alpha_mean+90*pi/180)*(+xrange)-sin(alpha_mean+90*pi/180)*(+yrange));
    x(3)=x1(2)+(cos(alpha_mean+90*pi/180)*(+xrange)-sin(alpha_mean+90*pi/180)*(-yrange));
    x(4)=x1(2)+(cos(alpha_mean+90*pi/180)*(-xrange)-sin(alpha_mean+90*pi/180)*(-yrange));
    x(5)=x1(2)+(cos(alpha_mean+90*pi/180)*(-xrange)-sin(alpha_mean+90*pi/180)*(+yrange));
    y=zeros(5,1);               % "y" coordinates of the vertices of the rectangle bounding the displacement region
    y(1)=y1(2)+(sin(alpha_mean+90*pi/180)*(-xrange)+cos(alpha_mean+90*pi/180)*(+yrange));
    y(2)=y1(2)+(sin(alpha_mean+90*pi/180)*(+xrange)+cos(alpha_mean+90*pi/180)*(+yrange));
    y(3)=y1(2)+(sin(alpha_mean+90*pi/180)*(+xrange)+cos(alpha_mean+90*pi/180)*(-yrange));
    y(4)=y1(2)+(sin(alpha_mean+90*pi/180)*(-xrange)+cos(alpha_mean+90*pi/180)*(-yrange));
    y(5)=y1(2)+(sin(alpha_mean+90*pi/180)*(-xrange)+cos(alpha_mean+90*pi/180)*(+yrange));
            
    
    % the fitted linear line:
    linFunc_x=linspace(xlimit(1), xlimit(2), 2);
    linFunc_y=initLine_linearFit.m*linFunc_x+initLine_linearFit.h;
    
    
    plot(linFunc_x, linFunc_y, 'color', 'c')             % plotting the fitted linear
    legend('off');
    hold on;
    drawArrow(x1,y1, 'linewidth',3, 'MaxHeadSize', 1, 'color','g');     % drawing arrow showing the central displacement
    hold on;
    plot(x, y, 'color', 'c');   % plotting the rectangle bounding the displacement region
    hold on
    viscircles([xCenter, yCenter], sig, 'Color', 'm', 'Linewidth', 1);  % plotting the circle showing the standard deviation of the Gaussian blur
    hold off;
    %disp(['Angle of fitted curve: ',num2str(angleOutConversion(alpha_mean)), '???'])
    
    % question dialog in which one can accept the current angle,
    % set a new one manually or quit from the program
    if automaticProcessBool
        choice='Accept parameters';
    else
        choice = questdlg({['Current angle of the initial line: ', num2str(angleOutConversion(alpha_mean)), '???'];...
            ['Current [x, y] displacement the rectangular region is: ', num2str(displacement), ' nm'];...
            ['Current length of the rotated rectangular region is: ', num2str(xrange), ' nm'];...
            ['Current height of the rotated rectangular region is: ', num2str(yrange), ' nm'];...
            ['Current standard deviation of the Gaussian blur is: ', num2str(sig), ' nm'];...
            ['Current maximum line-localization distance is: ', num2str(sampLineDistThreshold), ' nm'];...
            ['Current number of bins on the histogram is: ', num2str(nBins)];...
            ['Current polynomial degree is: ', num2str(polyDegree)];...
            ['Current max PSF size for filtering is: ', num2str(PSFmax)];...
            ['Current power for curve fitting on surface is: ' num2str(surfFittingPower)]}, ...
            'Parameters of the curve fitting region', 'Accept parameters','Set parameters manually','Quit', 'Accept parameters');
    end

    switch choice
        case 'Accept parameters'
            dlgBool=false;
        case 'Set parameters manually'
            prompt = {'Angle of the initial line: ', '[x, y] displacement the rectangular region [nm]:', 'Length of the rotated rectangular region [nm]:', 'Height of the rotated rectangular region [nm]:', 'Standard deviation of the Gaussian blur [nm]:', 'Maximum line-localization distance [nm]:', 'Number of bins on the histogram:', 'Degree of the polynomial to be fit:', 'maximal PSF size for filtering [pixel]:', 'Power minimizing the fitted curve along the surface:'}; % name of the input fields
            dlgTitle = 'Set a rectangular region';   % title of the dialog
            nLines = 1;     % one line for each input fields
            % the default values on the dialog
            defaultAns = {num2str(angleOutConversion(alpha_mean)), num2str(displacement), num2str(xrange), num2str(yrange), num2str(sig), num2str(sampLineDistThreshold), num2str(nBins), num2str(polyDegree), num2str(PSFmax), num2str(surfFittingPower)};
            % getting the new parameters
            answer = inputdlg(prompt,dlgTitle,nLines,defaultAns);  % the set "z" boundary values in a cell of strings
            alpha_mean=angleInConversion(str2num(char(answer{1})));
            displacement=str2num(answer{2});
            xrange=str2num(answer{3});
            yrange=str2num(answer{4});
            sig=str2num(answer{5});
            sampLineDistThreshold=str2num(answer{6});
            nBins=str2num(answer{7});
            polyDegree=str2num(answer{8});
            PSFmax=str2num(answer{9});
            surfFittingPower=str2num(answer{10});
            % the new parameters of the line
            initLine_linearFit.m=tan(alpha_mean+90*pi/180);
            % should go through the displaced center
            initLine_linearFit.h=yCenter-xCenter*tan(alpha_mean+90*pi/180)-displacement(1)*tan(alpha_mean+90*pi/180)+displacement(2);
            
            % Filtering based on the PSF size
            LOC_X_filtered = LOC_X(PSFSize<=PSFmax);
            LOC_Y_filtered = LOC_Y(PSFSize<=PSFmax);
        case 'Quit'
            dlgBool=false;
            error('Quitting was requested.');
    end
end
%close();

%% Export
% exporting the rectangular region parameters for the next line
% investigation
lineDiam_lastParams.alpha_mean=alpha_mean+90*pi/180;
lineDiam_lastParams.xrange=xrange;
lineDiam_lastParams.yrange=yrange;
lineDiam_lastParams.sig=sig;
lineDiam_lastParams.sampLineDistThreshold=sampLineDistThreshold;
lineDiam_lastParams.nBins=nBins;
lineDiam_lastParams.polyDegree=polyDegree;
lineDiam_lastParams.PSFmax=PSFmax;
lineDiam_lastParams.surfFittingPower=surfFittingPower;


end

