function [histDataPath] = rainSTORM_extras_lineDiameter_fit(histFullFileNames, clearing, model, averageDistBool, automaticSaveBool, automaticProcessBool)
% Extract parameters
initFitVals.gap.densFuncSize_init = model.densFuncSize_init;
initFitVals.gap.rimWidth = model.rimWidth;

% Clear or create the final table;
allParamsTable=[];

% checking the validity of some parameters
if strcmp(model.sampleType, 'gap')
    if ~strcmp(model.modelFuncType, 'Gap')
       error('For the "gap" structure you can only use the "gap" model function.') 
    end
elseif strcmp(model.sampleType, 'single line') || strcmp(model.sampleType, 'double lines')
    if strcmp(model.modelFuncType, 'Gap')
        error('For the "single band" or "double band" structures you can''t use the "gap" model function')
    elseif ~strcmp(model.modelFuncType,'Gaussian') && ~strcmp(model.modelFuncType,'Lorentzian') && ~strcmp(model.modelFuncType,'Rectangular') && ~strcmp(model.modelFuncType,'Disk') && ~strcmp(model.modelFuncType,'Circle')
        error('Unknown model function tpe');
    end
else
    error('Unknown sample tpe');
end

if model.iterGaussianConvBool
    if ~strcmp(model.modelFuncType, 'Gap')
       error('The iteration of Gaussian convolution ("iterGaussianConvBool=true") can onlyy be used with the "gap" structure.');
    end
end

%% Loop through the sections

if iscell(histFullFileNames)
    N_histDataFiles=numel(histFullFileNames);   % if multiple files were chosen, the names are stored in a cell array
else
    N_histDataFiles=1;      % if a single file was chosen, the name is stored in a string
end

for idxHist=1:N_histDataFiles
    % name of the current histogram data file
    currentHistDataFile=histFullFileNames{idxHist};
    disp(['Processing: ' currentHistDataFile]);

    % load the histogram data of the measured distances
    lineDiam_histData = load(currentHistDataFile, '-mat');
    figHandles = findobj('type', 'figure', '-not', 'name', 'Choosing the rectangular region of interest');
    close(figHandles);
    figHandles = findobj('type', 'figure', '-not', 'name', 'Fit curve along the highest localization density');
    close(figHandles);
    figHandles = findobj('type', 'figure', '-not', 'name', 'Histogram of distances.');
    close(figHandles);
    
    % preparing the distance data of the lines
    switch averageDistBool
        case true   % concatenate the distance data belonging to different lines
            lineDiam_histData_input(1).locLineDist=[lineDiam_histData.lineDiam_histData.locLineDist{:}];
            lineDiam_histData_input(1).ThompsonPrec=[lineDiam_histData.lineDiam_histData.ThompsonPrec{:}];
            lineDiam_histData_input(1).PSFSize=[lineDiam_histData.lineDiam_histData.PSFSize{:}];
            lineDiam_histData_input(1).histEdges=lineDiam_histData.lineDiam_histData.histEdges;
            lineDiam_histData_input(1).histCounts=lineDiam_histData.lineDiam_histData.histCounts;
            % Filtering
            FilterBool = ( (lineDiam_histData_input(1).PSFSize >= clearing.PSFSizeMin) & (lineDiam_histData_input(1).PSFSize <= clearing.PSFSizeMax) );
            lineDiam_histData_input(1).locLineDist = lineDiam_histData_input(1).locLineDist(FilterBool);
            lineDiam_histData_input(1).ThompsonPrec = lineDiam_histData_input(1).ThompsonPrec(FilterBool);
            lineDiam_histData_input(1).PSFSize = lineDiam_histData_input(1).PSFSize(FilterBool);
            
            % workaround to avoid error while saving
            [~,currentHistDataFileName,~]=fileparts(currentHistDataFile);
            matchedItem = regexp(currentHistDataFileName,'export\w*','match');
            if ~isempty(matchedItem)
                lineDiam_histData.lineDiam_histData.export_name=matchedItem;
            else
                lineDiam_histData.lineDiam_histData.export_name=currentHistDataFileName;
            end
            lineDiam_histData.lineDiam_histData.SectionRawDataFullFilename=currentHistDataFileName;
        otherwise   % place the distance data in a structure array to handle them seperately for the different lines
            NLines=length(lineDiam_histData.lineDiam_histData.locLineDist);
            lineDiam_histData_input=repmat(struct('locLineDist', [], 'ThompsonPrec', [], 'PSFSize', [], 'histEdges', [], 'histCounts', []), NLines, 1);
            for idxLine=1:NLines
                lineDiam_histData_input(idxLine).locLineDist=[lineDiam_histData.lineDiam_histData.locLineDist{idxLine}];
                lineDiam_histData_input(idxLine).ThompsonPrec=[lineDiam_histData.lineDiam_histData.ThompsonPrec{idxLine}];
                lineDiam_histData_input(idxLine).PSFSize=[lineDiam_histData.lineDiam_histData.PSFSize{idxLine}];
                lineDiam_histData_input(idxLine).histEdges{1}=lineDiam_histData.lineDiam_histData.histEdges{idxLine};
                lineDiam_histData_input(idxLine).histCounts{1}=lineDiam_histData.lineDiam_histData.histCounts{idxLine};
                % Filtering
                FilterBool = ( (lineDiam_histData_input(idxLine).PSFSize >= clearing.PSFSizeMin) & (lineDiam_histData_input(idxLine).PSFSize <= clearing.PSFSizeMax) );
                lineDiam_histData_input(idxLine).locLineDist = lineDiam_histData_input(idxLine).locLineDist(FilterBool);
                lineDiam_histData_input(idxLine).ThompsonPrec = lineDiam_histData_input(idxLine).ThompsonPrec(FilterBool);
                lineDiam_histData_input(idxLine).PSFSize = lineDiam_histData_input(idxLine).PSFSize(FilterBool);
            end
    end
    
    % open the dialog to let the user set the fitting parameters
    if ~ automaticProcessBool
        ThompsonPrec_toDialog=[lineDiam_histData.lineDiam_histData.ThompsonPrec{:}];
        
        % input dialog for the fit parameters
        dialogOutput=lineDiameter_fit_inputDialog( ThompsonPrec_toDialog, model.sampleType, model.modelFuncType, model.linkerRad, model.locPrecision, model.multiPrecBool );
        model.modelFuncType=dialogOutput.modelFuncType;
        model.linkerRad=dialogOutput.linkerLength;
        precBounds=dialogOutput.precBounds;
        model.locPrecision=dialogOutput.singlePrecValue;
        model.multiPrecBool=dialogOutput.multiPrecBool;
    % defining here the defaut fitting parameters
    else
        precBounds=[-Inf Inf];
    end
    
    [histDataPath, histDataName, ~]=fileparts(currentHistDataFile);    
    
    % call the line width fitting on the distance data
    N_lineDiam_histData_input=length(lineDiam_histData_input);
    for idxLine=1:N_lineDiam_histData_input
        [fittedParamsTable, fittedHistFigure]=lineDiameter_fit( lineDiam_histData_input(idxLine), model.curveFitTool, ...
            model.linkerType, model.sampleType, model.modelFuncType, model.linkerRad, precBounds, model.locPrecision, model.multiPrecBool, initFitVals, model.iterGaussianConvBool );
        fittedParamsTable.histDataName = {histDataName};
        fittedParamsTable.export_name = lineDiam_histData.lineDiam_histData.export_name;
        fittedParamsTable.SectionRawDataFullFilename = lineDiam_histData.lineDiam_histData.SectionRawDataFullFilename;
        
        if ~automaticSaveBool
            % draw the figure of the result
            figure(fittedHistFigure)
            
            choice = questdlg(...
                'Do you want to save this fitted line diameter data?',...
                'Fitted line diameter dialog',...
                'Save', 'Discard', 'Quit', 'Save');
            
            switch choice
                case 'Save'
                % saving the result manually
                    % attaching these strings to the output files
                    if N_lineDiam_histData_input==1
                        imageName=['_', 'fittedWidthImage'];
                        figureName=['_', 'fittedWidthFigure'];
                        paramsName=['_', 'fittedParams'];
                    elseif N_lineDiam_histData_input>1
                        imageName=['_', 'fittedWidthImage', '_', num2str(idxLine)];
                        figureName=['_', 'fittedWidthFigure', '_', num2str(idxLine)];
                        paramsName=['_', 'fittedParams', '_', num2str(idxLine)];
                    end
                    
                    [fileName, pathName]=uiputfile(...
                        '.mat',...
                        'Save the fitted parameters',...
                        fullfile(histDataPath, [histDataName, paramsName, '.mat']));
                    fullFileName=fullfile(pathName, fileName);
                    
                    % disable it if you do not want to save the figerus
                    % when saving the central line - localization distances manually 
                    if true
                        % saving the resultant figure depicting the fit as an image
                        mkdir(pathName, 'fittedWidthImage');
                        imageFullName=fullfile(pathName, ['fittedWidthImage', filesep, histDataName, imageName, '.jpg']);
                        saveas(fittedHistFigure, imageFullName, 'jpg');
                        % saving the resultant figure depicting the fit as a Matlab figure
                        mkdir(pathName, 'fittedWidthFigure');
                        figureFullName=fullfile(pathName, ['fittedWidthFigure', filesep, histDataName, figureName, '.fig']);
                        savefig(fittedHistFigure, figureFullName);
                        % saving the fitted parameters in a "mat" data file
                        save(fullFileName, 'fittedParamsTable', '-mat');
                    end
                    
                    if isempty(allParamsTable)
                        allParamsTable = fittedParamsTable;
                    else
                        allParamsTable= tableReplacement.concatenate(allParamsTable, fittedParamsTable);
                    end
                case 'Discard'
                    % do nothing
                case 'Quit'
                    error('Quitting was requested.');
            end
        else
        % saving the result automatically
            % attaching these strings to the output files
            if N_lineDiam_histData_input==1
                imageName=['_', 'fittedWidthImage'];
                figureName=['_', 'fittedWidthFigure'];
                paramsName=['_', 'fittedParams'];
            elseif N_lineDiam_histData_input>1
                imageName=['_', 'fittedWidthImage', '_', num2str(idxLine)];
                figureName=['_', 'fittedWidthFigure', '_', num2str(idxLine)];
                paramsName=['_', 'fittedParams', '_', num2str(idxLine)];
            end
            % saving the resultant figure depicting the fit as an image
            mkdir(histDataPath, 'fittedWidthImage');
            imageFullName=fullfile(histDataPath, ['fittedWidthImage', filesep, histDataName, imageName, '.jpg']);
            saveas(fittedHistFigure, imageFullName, 'jpg');
            % saving the resultant figure depicting the fit as a Matlab figure
            mkdir(histDataPath, 'fittedWidthFigure');
            figureFullName=fullfile(histDataPath, ['fittedWidthFigure', filesep, histDataName, figureName, '.fig']);
            savefig(fittedHistFigure, figureFullName);
            % saving the fitted parameters in a "mat" data file
            mkdir(histDataPath, 'fittedParams');
            fittedParamsFullName=fullfile(histDataPath, ['fittedParams', filesep, histDataName, paramsName, '.mat']);
            save(fittedParamsFullName, 'fittedParamsTable', '-mat');
            
            if idxHist==1
                allParamsTable = fittedParamsTable;
            else
                allParamsTable= tableReplacement.concatenate(allParamsTable, fittedParamsTable);
            end
            
        end
        close(fittedHistFigure) 
    end
end
% write the fitted parameters into a single file
if ~isempty(allParamsTable)
    mkdir(histDataPath, 'fittedParams');
    tableReplacement.write(allParamsTable,fullfile(histDataPath, ['fittedParams', filesep, 'allFittedParams.csv']))
end
end